"use strict";

var _react = _interopRequireDefault(require("react"));

var _faker = _interopRequireDefault(require("faker"));

var _errors = require("../../utilities/errors");

var _testUtilities = require("../../test-utilities");

var _Text = require("../Text");

var _Tooltip = require("../Tooltip");

var _InlineError = require("../InlineError");

var _VisuallyHidden = require("../VisuallyHidden");

var _TextField = require("./TextField");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var defaultProps = {
  id: 'text_field',
  label: 'First name'
};
describe('<TextField />', function () {
  it('renders an input field with basic props', function () {
    var id = _faker["default"].random.alphaNumeric();

    var name = _faker["default"].random.word();

    var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
      id: id,
      name: name
    })));
    expect(textField).toContainReactComponent('input', {
      id: id,
      name: name
    });
  });
  it('renders an input field with a prefilled value when the value prop is provided', function () {
    var value = 'Shopifolk';
    var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
      value: value
    })));
    expect(textField).toContainReactComponent('input', {
      value: value
    });
  });
  it('renders an error message when the error prop is provided', function () {
    var error = 'Enter a first name';

    var id = _faker["default"].random.alphaNumeric();

    var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
      id: id,
      error: error
    })));
    expect(textField).toContainReactComponent(_InlineError.InlineError, {
      controlID: id,
      children: error
    });
  });
  it('renders a tooltip when the tooltip prop is provided', function () {
    var label = 'More information';
    var content = 'In case we need to contact you about your order';
    var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
      tooltip: {
        label: label,
        content: content
      }
    })));
    expect(textField).toContainReactComponent(_Tooltip.Tooltip, {
      content: content
    });
    expect(textField.find(_VisuallyHidden.VisuallyHidden)).toContainReactText(label);
  });
  it('renders a textarea when the multiline prop is provided', function () {
    var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
      multiline: true
    })));
    expect(textField).toContainReactComponent('textarea');
    expect(textField).not.toContainReactComponent('input');
  });
  it('calls an onFocus callback when the input gains focus', function () {
    var onFocus = jest.fn();
    var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
      onFocus: onFocus
    })));
    textField.find('input').trigger('onFocus');
    expect(onFocus).toHaveBeenCalled();
  });
  it('calls an onBlur callback when the input loses focus', function () {
    var onBlur = jest.fn();
    var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
      onBlur: onBlur
    })));
    textField.find('input').trigger('onBlur', {
      currentTarget: {
        value: ''
      }
    });
    expect(onBlur).toHaveBeenCalled();
  });
  it('calls an onInput callback on input change with the current value', function () {
    var onInput = jest.fn();
    var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
      onInput: onInput
    })));

    var value = _faker["default"].random.alphaNumeric();

    triggerChange(textField, value);
    expect(onInput).toHaveBeenCalledWith(value);
  });
  describe('type', function () {
    it('is applied to the input', function () {
      var type = 'email';
      var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
        type: type
      })));
      expect(textField).toContainReactComponent('input', {
        type: type
      });
    });
    it('normalizes the telephone type to tel', function () {
      var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
        type: "telephone"
      })));
      expect(textField).toContainReactComponent('input', {
        type: 'tel'
      });
    });
  });
  describe('onChange', function () {
    it('does not call onChange when the value changes but is not committed, but updates the value in the input', function () {
      var onChange = jest.fn();
      var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
        value: "",
        onChange: onChange
      })));

      var value = _faker["default"].random.alphaNumeric();

      triggerChange(textField, value);
      expect(onChange).not.toHaveBeenCalled();
      expect(textField).toContainReactComponent('input', {
        value: value
      });
    });
    it('does not call onChange when the value changes and is committed, but matches the current prop value', function () {
      var value = _faker["default"].random.alphaNumeric();

      var onChange = jest.fn();
      var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
        value: value,
        onChange: onChange
      })));
      triggerChange(textField, '');
      triggerChange(textField, value);
      textField.find('input').trigger('onBlur', {
        currentTarget: {
          value: value
        }
      });
      expect(onChange).not.toHaveBeenCalled();
    });
    it('calls onChange when the value changes and is committed, and does not match the current prop value', function () {
      var onChange = jest.fn();
      var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
        value: "",
        onChange: onChange
      })));

      var value = _faker["default"].random.alphaNumeric();

      triggerChange(textField, value);
      textField.find('input').trigger('onBlur', {
        currentTarget: {
          value: value
        }
      });
      expect(onChange).toHaveBeenCalledWith(value);
    });
  });
  describe('label', function () {
    it('renders a placeholder by default', function () {
      var label = 'Email';
      var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
        label: label
      })));
      expect(textField).toContainReactComponent('input', {
        placeholder: label
      });
    });
    it('hides the placeholder once a value is entered', function () {
      var label = 'Email';
      var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
        label: label
      })));
      triggerChange(textField, _faker["default"].random.alphaNumeric());
      expect(textField).not.toContainReactComponent('input', {
        placeholder: label
      });
    });
    it('hides the placeholder once a value is entered and removed', function () {
      var label = 'Email';
      var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
        label: label
      })));
      triggerChange(textField, 'snowdevil@shopify.com');
      triggerChange(textField, '');
      expect(textField).not.toContainReactComponent('input', {
        placeholder: label
      });
    });
    it.todo('calls `labelled.onFocus when focusing the textfield`');
    it.todo('calls `labelled.onBlur when removing focus from the textfield`');
    it.todo('calls `labelled.onChange with a value when typing a value in the field`');
    it.todo('calls `labelled.onChange with an empty value when typing and the field is empty`');
  });
  describe('autofocus', function () {
    var rawCreateElement = document.createElement;
    var focusSpy = jest.fn();
    var createElementSpy = jest.spyOn(document, 'createElement');
    beforeEach(function () {
      createElementSpy.mockImplementation(function () {
        for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        var result = rawCreateElement.apply(document, args);

        if (args[0].toLocaleLowerCase() === 'input') {
          result.focus = focusSpy;
        }

        return result;
      });
    });
    afterEach(function () {
      focusSpy.mockReset();
    });
    afterAll(function () {
      createElementSpy.mockRestore();
    });
    it('does not focus inputs by default', function () {
      (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, defaultProps));
      expect(focusSpy).not.toHaveBeenCalled();
    });
    it('focuses on the input when autofocus is specified', function () {
      (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
        autofocus: true
      })));
      expect(focusSpy).toHaveBeenCalledTimes(1);
    });
  });
  describe('accessibilityDescription', function () {
    it('omits the hidden description element when accessibilityDescription is not specified', function () {
      var id = _faker["default"].random.alphaNumeric();

      var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
        id: id
      })));
      expect(textField).not.toContainReactComponent(_Text.Text, {
        id: "".concat(id, "-description")
      });
    });
    it('renders a hidden element containing accessibilityDescription text', function () {
      var id = _faker["default"].random.alphaNumeric();

      var description = _faker["default"].random.words();

      var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
        id: id,
        accessibilityDescription: description
      })));
      var descriptionElement = textField.find(_Text.Text, {
        id: "".concat(id, "-description")
      });
      expect(descriptionElement).toContainReactText(description);
    });
    it('associates the input with the hidden accessibilityDescription element', function () {
      var id = _faker["default"].random.alphaNumeric();

      var description = _faker["default"].random.words();

      var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
        id: id,
        accessibilityDescription: description
      })));
      expect(textField).toContainReactComponent('input', {
        id: id,
        'aria-describedby': "".concat(id, "-description")
      });
    });
    it('includes error element id in describedy associations', function () {
      var id = _faker["default"].random.alphaNumeric();

      var description = _faker["default"].random.words();

      var error = _faker["default"].random.words();

      var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
        id: id,
        accessibilityDescription: description,
        error: error
      })));
      expect(textField).toContainReactComponent('input', {
        id: id,
        'aria-describedby': "".concat(id, "-description ").concat((0, _errors.errorId)(id))
      });
    });
  });
  describe('forward refs', function () {
    it('passes input element to ref functions', function () {
      var id = _faker["default"].random.alphaNumeric();

      var refCallback = jest.fn();
      var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
        id: id,
        ref: refCallback
      })));
      expect(refCallback).toHaveBeenCalledWith(textField.find('input').instance);
    });
    it('assigns input element to ref.current on ref instances', function () {
      var id = _faker["default"].random.alphaNumeric();

      var fakeRef = {
        current: null
      };
      var textField = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react["default"].createElement(_TextField.TextField, Object.assign({}, defaultProps, {
        id: id,
        ref: fakeRef
      })));
      expect(fakeRef.current).toBe(textField.find('input').instance);
    });
  });
});

function triggerChange(textField, value) {
  // For some reason, the testing library seems to pick up some sort of internal
  // Preact logic. This should be triggering the `onChange` callback, which is definitely
  // defined, but before the test library wraps it all up, Preact has normalized
  // `onChange` to `oninput`.
  textField.find('input').trigger('oninput', {
    currentTarget: {
      value: value
    }
  });
}